@@ -2,7 +2,7 @@
 
     Compatibility name file for  devfsd  (build compatibility names).
 
-    Copyright (C) 1998-2001  Richard Gooch
+    Copyright (C) 1998-2002  Richard Gooch
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -48,9 +48,16 @@
     Updated by      Richard Gooch   6-NOV-2001: Updated compatibility entry
   support for fixed Computone serial driver names.
 
-    Last updated by Richard Gooch   8-NOV-2001: Updated compatibility entry
+    Updated by      Richard Gooch   8-NOV-2001: Updated compatibility entry
   support for pending Cyclades serial driver names.
 
+    Updated by      Richard Gooch   19-JAN-2002: Fixed <write_old_sd_name> when
+  there are more than 26 SCSI discs.
+
+    Last updated by Richard Gooch   20-JAN-2002: Fixed <get_old_name> to ignore
+  new compatibility names for IDE devices. Consolidated SCSI code. Consolidated
+  IDE code.
+
 
 */
 #include <linux/major.h>
@@ -149,68 +156,59 @@ const char *get_old_name (const char *devname, unsigned int namelen,
 	sprintf (buffer, "sbpcd%u", minor);
 	compat_name = buffer;
     }
-    else if ( (strncmp (devname, "scsi/", 5) == 0) &&
-	      (strcmp (devname + namelen - 7, "generic") == 0) )
-    {
-	sprintf (buffer, "sg%u", minor);
-	compat_name = buffer;
-    }
-    else if ( (strncmp (devname, "scsi/", 5) == 0) &&
-	      (strncmp (ptr = (strrchr (devname, '/') + 1), "mt", 2)
-	       == 0) )
-    {
-	char mode = ptr[2];
+    else if (strncmp (devname, "scsi/", 5) == 0)
+    {   /*  All SCSI devices  */
+	if (strcmp (devname + namelen - 7, "generic") == 0)
+	{
+	    sprintf (buffer, "sg%u", minor);
+	    compat_name = buffer;
+	}
+	else if (strncmp (ptr = (strrchr (devname, '/') + 1), "mt", 2) == 0)
+	{
+	    char mode = ptr[2];
 
-	if (mode == 'n') mode = '\0';
-	sprintf (buffer, "nst%u%c", minor & 0x1f, mode);
-	compat_name = buffer;
-	if (devname[namelen - 1] != 'n') ++compat_name;
+	    if (mode == 'n') mode = '\0';
+	    sprintf (buffer, "nst%u%c", minor & 0x1f, mode);
+	    compat_name = buffer;
+	    if (devname[namelen - 1] != 'n') ++compat_name;
+	}
+	else if (strcmp (devname + namelen - 2, "cd") == 0)
+	{
+	    sprintf (buffer, "sr%u", minor);
+	    compat_name = buffer;
+	}
+	else if (strcmp (devname + namelen - 4, "disc") == 0)
+	    compat_name = write_old_sd_name (buffer, major, minor, "");
+	else if (strncmp (ptr = (strrchr (devname, '/') + 1), "part", 4) == 0)
+	    compat_name = write_old_sd_name (buffer, major, minor, ptr + 4);
+	return (compat_name);
     }
-    else if ( (strncmp (devname, "ide/", 4) == 0) &&
-	      (strncmp (ptr = (strrchr (devname, '/') + 1), "mt", 2)
-	       == 0) )
-    {
-	sprintf (buffer, "%sht%d", ptr + 2, minor & 0x7f);
-	compat_name = buffer;
-    }
-    else if ( (strncmp (devname, "scsi/", 5) == 0) &&
-	      (strcmp (devname + namelen - 2, "cd") == 0) )
-    {
-	sprintf (buffer, "sr%u", minor);
-	compat_name = buffer;
-    }
-    else if ( (strncmp (devname, "scsi/", 5) == 0) &&
-	      (strcmp (devname + namelen - 4, "disc") == 0) )
-    {
-	compat_name = write_old_sd_name (buffer, major, minor, "");
-    }
-    else if ( (strncmp (devname, "scsi/", 5) == 0) &&
-	      (strncmp (ptr = (strrchr (devname, '/') +1), "part", 4)
-	       == 0) )
-    {
-	compat_name = write_old_sd_name (buffer, major, minor, ptr + 4);
-    }
-    else if ( (strncmp (devname, "ide/", 4) == 0) &&
-	      (strcmp (devname + namelen - 4, "disc") == 0) )
-    {
-	sprintf ( buffer, "hd%c",
-		  get_old_ide_name (major, minor) );
-	compat_name = buffer;
-    }
-    else if ( (strncmp (devname, "ide/", 4) == 0) &&
-	      (strncmp (ptr = (strrchr (devname, '/') +1), "part", 4)
-	       == 0) )
-    {
-	sprintf (buffer, "hd%c%s",
-		 get_old_ide_name (major, minor), ptr + 4);
-	compat_name = buffer;
-    }
-    else if ( (strncmp (devname, "ide/", 4) == 0) &&
-	      (strcmp (devname + namelen - 2, "cd") == 0) )
-    {
-	sprintf ( buffer, "hd%c",
-		  get_old_ide_name (major, minor) );
-	compat_name = buffer;
+    else if (strncmp (devname, "ide/host", 8) == 0)
+    {   /*  All IDE devices  */
+	if (strncmp (ptr = (strrchr (devname, '/') + 1), "mt", 2) == 0)
+	{
+	    sprintf (buffer, "%sht%d", ptr + 2, minor & 0x7f);
+	    compat_name = buffer;
+	}
+	else if (strcmp (devname + namelen - 4, "disc") == 0)
+	{
+	    sprintf ( buffer, "hd%c",
+		      get_old_ide_name (major, minor) );
+	    compat_name = buffer;
+	}
+	else if (strncmp (ptr = (strrchr (devname, '/') + 1), "part", 4) == 0)
+	{
+	    sprintf (buffer, "hd%c%s",
+		     get_old_ide_name (major, minor), ptr + 4);
+	    compat_name = buffer;
+	}
+	else if (strcmp (devname + namelen - 2, "cd") == 0)
+	{
+	    sprintf ( buffer, "hd%c",
+		      get_old_ide_name (major, minor) );
+	    compat_name = buffer;
+	}
+	return (compat_name);
     }
     else if (strncmp (devname, "vcc/", 4) == 0)
     {
@@ -305,7 +303,7 @@ static char *write_old_sd_name (char *buffer,
 	if (disc_index < 26)
 	    sprintf (buffer, "sd%c%s", 'a' + disc_index, part);
 	else sprintf (buffer, "sd%c%c%s",
-		      'a' + (disc_index / 26) - 1, 'a' + disc_index, part);
+		      'a' + (disc_index / 26) - 1, 'a' + disc_index % 26,part);
 	return (buffer);
     }
     return (NULL);
